@@ -16,12 +16,10 @@ class Logo(Renderer):
         super(Logo, self).__init__()
         self.version = version
 
-    def draw(self, echo):
-        self._curr_row = 0
+    def draw(self):
         for line in LOGO:
-            echo(self.x, self.y + self._curr_row, line)
-            self._curr_row += 1
+            self.echo(line)
 
         version_text = self.version
         version_x_pos = round(self.width / 2 - len(version_text) / 2 + self.x)
-        echo(version_x_pos, self.y + self._curr_row, version_text)
+        self.echo(version_text, x=version_x_pos)
