@@ -1,17 +1,16 @@
-import asyncio
-
 import os
 import sys
-from typing import List
+import asyncio
 import tempfile
+from typing import List
 from subprocess import call
 
-from blessed import Terminal
 from colored import attr
+from blessed import Terminal
 
 from a9s.components.custom_string import String
 from a9s.components.renderer import Renderer
-
+from a9s.components.utils import FPS
 
 EDITOR = os.environ.get('EDITOR', 'vim')
 
@@ -21,12 +20,19 @@ def flush():
 
 
 class App:
-    def __init__(self):
+    def __init__(self, render_list):
         self.term = Terminal()
-        self.render_list = []
+        self.render_list: list[Renderer] = render_list
         self.should_run = True
 
+        self.buffer = []
+        self.on_resize()
+
+    def on_resize(self, *args):
         self.buffer = [[' ' for x in range(self.term.width)] for y in range(self.term.height)]
+        for rendered in self.render_list:
+            rendered.onresize()
+        sys.stdout.write(self.term.clear)
 
     def dump_to_screen(self):
         for line_num, line in enumerate(self.buffer):
@@ -34,9 +40,6 @@ class App:
 
         sys.stdout.write(attr("reset"))
 
-    def resize(self):
-        self.buffer = [[' ' for x in range(self.term.width)] for y in range(self.term.height)]
-
     def echo(self, x, y, to_print):
         if isinstance(to_print, str):
             to_print = String(to_print)
@@ -50,10 +53,6 @@ class App:
             actual_x = x + i
             line[actual_x] = str(char)
 
-    def clear(self):
-        sys.stdout.write(self.term.clear)
-        self.buffer = [[' ' for x in range(self.term.width)] for y in range(self.term.height)]
-
     def add_to_render(self, renderer: Renderer):
         self.render_list.append(renderer)
 
@@ -66,29 +65,38 @@ class App:
             tf.write(initial_text.encode())
             tf.flush()
             call([EDITOR, tf.name])
-
-            # do the parsing with `tf` using regular File operations.
-            # for instance:
             tf.seek(0)
             return tf.read().decode()
 
-    async def interactive_run(self):
+    async def draw_loop(self):
         with self.term.fullscreen():
+            while self.should_run:
+                with self.term.hidden_cursor():
+                    await asyncio.gather(asyncio.sleep(FPS), *[renderer.update_data() for renderer in self.render_list])
+                    for renderer in self.render_list:
+                        renderer.render(self.echo)
+
+                    self.dump_to_screen()
+                    flush()
+
+    async def input_loop(self):
+        def _blocking_input_loop():
             with self.term.cbreak(), self.term.raw():
                 while self.should_run:
-                    # self.echo(0, 0, self.term.clear)
-                    with self.term.hidden_cursor():
-                        await asyncio.gather(*[renderer.update_data() for renderer in self.render_list])
-                        for renderer in self.render_list:
-                            renderer.set_echo_func(self.echo)
-                            renderer.draw()
-                            renderer.fill_empty()
+                    key = self.term.inkey(timeout=FPS)
+                    if key in [chr(3)]:
+                        self.should_run = False
+                        break
 
-                        self.dump_to_screen()
-                        flush()
+                    self.on_tick(key)
 
-                        key = self.term.inkey(timeout=0.1)
-                        if key in [chr(3)]:
-                            break
+        await asyncio.create_task(asyncio.to_thread(_blocking_input_loop))
 
-                        yield key
+    def on_tick(self, key):
+        pass
+
+    async def run(self):
+        for renderer in self.render_list:
+            renderer.initialize()
+
+        await asyncio.gather(self.input_loop(), self.draw_loop())
